@@ -43,8 +43,8 @@ transfer_done_targz_idle_cb (gpointer data)
 	gchar *path = data;
 
 	/* this should be something more clever and nonblocking */
-	command = g_strdup_printf ("sh -c 'gzip -d -c < \"%s\" | tar xf - -C \"%s/.themes\"'",
-				   path, g_get_home_dir ());
+	command = g_strdup_printf ("sh -c 'cd \"%s/.themes\"; gzip -d -c < \"%s\" | tar xf -'",
+				   g_get_home_dir (), path);
 	if (g_spawn_command_line_sync (command, NULL, NULL, &status, NULL) && status == 0)
 		gnome_vfs_unlink (path);
 	g_free (command);
@@ -61,8 +61,8 @@ transfer_done_tarbz2_idle_cb (gpointer data)
 	gchar *path = data;
 
 	/* this should be something more clever and nonblocking */
-	command = g_strdup_printf ("sh -c 'bzip2 -d -c < \"%s\" | tar xf - -C \"%s/.themes\"'",
-				   path, g_get_home_dir ());
+	command = g_strdup_printf ("sh -c 'cd \"%s/.themes\"; bzip2 -d -c < \"%s\" | tar xf -'",
+				   g_get_home_dir (), path);
 	if (g_spawn_command_line_sync (command, NULL, NULL, &status, NULL) && status == 0)
 		gnome_vfs_unlink (path);
 	g_free (command);
@@ -502,6 +502,7 @@ do_get_file_info(GnomeVFSMethod *method,
 		 GnomeVFSContext *context)
 {
 	gchar *path = NULL;
+	GnomeThemeMetaInfo *theme;
 	
 	path = get_path_from_uri(uri);
 	if (!path)
@@ -529,7 +530,6 @@ do_get_file_info(GnomeVFSMethod *method,
 	}
 	else {
 		g_free (path);
-		GnomeThemeMetaInfo *theme;
 		
 		theme = theme_meta_info_find (uri);
 		if (theme)
